/*   BEM = Batch Equilibrium Model (calculates chemical equilibira)
     Copyright (C) 1996  J. Prenzel (jprenze@gwdg.de)
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */

#include <stdio.h>
#include <math.h>

#include "bem.h"


/*------------------------------------------------ cttmng -----------------*/

int cttmng(double delta[]){
  int i,k,rc;
  double old[MAXELT+MAXIEX+MAXGAS],alfold;


  Cl->alf=0;
  Cl->it2cnt=0;
  old[0]=Cl->theta;
  if(old[0]+delta[0] <0.0){
    bemerr(42,"theta",Cl->id,"",Cl->step);
    return -32;
  }
  old[1]=Cl->sigma;
  if(old[1]+delta[1] <0.0){
    bemerr(42,"sigma",Cl->id,"",Cl->step);
    return -32;
  }
  for(i=2;i<Cl->nelt;i++){
    old[i]=Cl->el[i]->ctot;
    if(old[i]+delta[i] <0.0){
      bemerr(42,Cl->el[i]->name,Cl->id,"",Cl->step);
      return -32;
    }
  }
  k=Cl->nelt;
  if(Cl->niex > 0){
    old[k]=Cl->xc->xcap;
    if(old[k]+delta[k] <0.0){
      bemerr(42,"cec",Cl->id,"",Cl->step);
      return -32;
    }
    k++;
  }
  for(i=0;i<Cl->ngas;i++){
    old[k]=Cl->gs[i]->pfix;
    if(old[k]+delta[k] <0.0){
      bemerr(42,Cl->gs[i]->name,Cl->id,"",Cl->step);
      return -32;
    }
    k++;
  }

/*  for(i=0;i<k;i++){
    printf("/// cttmng %d %g %g\n",i,old[i],delta[i]);
  } */


  while(1){
    alfold=Cl->alf;
    Cl->alf += Cl->dalf;
    if(Cl->alf >1.) Cl->alf=1.;

    Cl->theta=old[0] + Cl->alf*delta[0];
    Cl->sigma=old[1] + Cl->alf*delta[1];
    for(i=2;i<Cl->nelt;i++){
      Cl->el[i]->ctot = old[i] + Cl->alf*delta[i];
    }
    k=Cl->nelt;
    if(Cl->niex > 0){
      Cl->xc->xcap = old[k] + Cl->alf*delta[k];
      k++;
    }
    for(i=0;i<Cl->ngas;i++){
      Cl->gs[i]->pfix = old[k] + Cl->alf*delta[k];
      k++;
    }
    rc=sltmng();

    if(Cl->alf >= 1. && rc == 0){
      if(Cl->itrace > 0){
	prpout(Cl);
	fprintf(Lgf.fil,"cell %s, step %d, calls %d, pH = %8.4g\n"
                ,Cl->id,Cl->step,Cl->it2cnt,Cl->el[0]->pact);
      }
      return 0;
    }
    if(Cl->step == 0 ){
      bemerr(40,"","","",0);
      return 5;
    }
    if(Cl->it2cnt > Cl->it2max) return 5; 
                                        /* Schrittweiten (Homotopie) */
    if(rc != 0){
      Cl->alf = alfold;
      Cl->dalf *= 0.1;
    }
    else{
      Cl->dalf *= 2.;
      if(Cl->dalf > 1.) Cl->dalf=1.;
    }

  }
  return 0;
}

